@@ -1,26 +1,8 @@
-# ============================================================================
-# The MIT License
-#
-# Copyright (c) 2018 Infineon Technologies AG
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE
-# ============================================================================
+#!/usr/bin/env python
+"""This module defines chip and object related base classes of the optigatrust package """
+# pylint: disable=too-many-lines
+# for the time-being this file is large, a major restructure is required
+
 from collections import namedtuple
 import warnings
 from ctypes import c_int, c_ushort, c_ubyte, POINTER, byref, memmove
@@ -45,7 +27,6 @@ __all__ = [
 
 
 UID = namedtuple("UID", "cim_id platform_id model_id rommask_id chip_type batch_num x_coord y_coord fw_id fw_build")
-_optiga_cddl = None
 
 
 def _lookup_optiga(api):
@@ -54,14 +35,14 @@ def _lookup_optiga(api):
     api.exp_optiga_util_read_metadata.argtypes = c_ushort, POINTER(c_ubyte), POINTER(c_ushort)
     api.exp_optiga_util_read_metadata.restype = c_int
 
-    d = (c_ubyte * 1700)()
+    c_d = (c_ubyte * 1700)()
     c_dlen = c_ushort(1700)
 
-    ret = api.exp_optiga_util_read_data(c_ushort(0xE0C2), 0, d, byref(c_dlen))
+    ret = api.exp_optiga_util_read_data(c_ushort(0xE0C2), 0, c_d, byref(c_dlen))
 
-    if ret == 0 and not all(_d == 0 for _d in list(bytes(d))):
+    if ret == 0 and not all(_d == 0 for _d in list(bytes(c_d))):
         data = (c_ubyte * c_dlen.value)()
-        memmove(data, d, c_dlen.value)
+        memmove(data, c_d, c_dlen.value)
         _bytes = bytearray(data)
     else:
         _bytes = bytearray(0)
@@ -70,25 +51,30 @@ def _lookup_optiga(api):
 
     # Trust M1 or Charge
     if _fw_build in {0x501, 0x624, 0x751, 0x802, 0x809}:
-        ret = api.exp_optiga_util_read_metadata(c_ushort(0xe0fc), d, byref(c_dlen))
+        ret = api.exp_optiga_util_read_metadata(c_ushort(0xe0fc), data, byref(c_dlen))
         if ret != 0:
             # it means that we work with OPTIGA Trust Charge
-            return m1, 'OPTIGA™ Trust Charge V1 (SLS32AIA020U2/3)'
-        else:
-            return m1, 'OPTIGA™ Trust M V1 (SLS32AIA010MH/S)'
+            return charge, 'OPTIGA™ Trust Charge V1 (SLS32AIA020U2/3)'
+
+        return m1, 'OPTIGA™ Trust M V1 (SLS32AIA010MH/S)'
     # Trust M2 ID2 or M3
     if _fw_build in {0x2440}:
-        ret = api.exp_optiga_util_read_metadata(c_ushort(0xe0f1), d, byref(c_dlen))
+        ret = api.exp_optiga_util_read_metadata(c_ushort(0xe0f1), data, byref(c_dlen))
         if ret != 0:
             # it means that we work with OPTIGA Trust M2 ID2
-            return m3, 'OPTIGA™ Trust M2 ID2 (SLS32AIA010I2/3)'
-        else:
-            return m3, 'OPTIGA™ Trust M V3 (SLS32AIA010ML/K)'
+            return m2id2, 'OPTIGA™ Trust M2 ID2 (SLS32AIA010I2/3)'
 
-    elif _fw_build in {0x510, 0x715, 0x1048, 0x1112, 0x1118}:
+        return m3, 'OPTIGA™ Trust M V3 (SLS32AIA010ML/K)'
+
+    if _fw_build in {0x510, 0x715, 0x1048, 0x1112, 0x1118}:
         return x, 'OPTIGA™ Trust X (SLS32AIA020X2/4)'
 
+    return None, ''
 
+
+# pylint: disable=too-many-instance-attributes disable=no-self-use
+# 14 is a reasonable amount as it represents a real amount of properties.
+# Parameters are related to the chip and don't use self, but shouldn't be distinct functions
 class Chip:
     """
     A class used to represent the whole OPTIGA Trust Chip
@@ -202,7 +188,7 @@ class Chip:
             raise ValueError(
                 'Wrong lifecycle state. Expected {0}, your provided {1}'.format(lifecycle_states, val)
             )
-        for code, state in lifecycle_states.items():
+        for _, state in lifecycle_states.items():
             if state == val:
                 Object(0xe0c0).write(bytes(state))
 
@@ -260,6 +246,38 @@ class Chip:
         config += bytes([delayed_sec_sync])
         Object(0xe0c9).write(config)
 
+    def protected_update(self, manifest, fragments):
+        """
+        This function helps to use the protected update feature of the chip
+
+        :param manifest:
+            The Manifest is a top level construct that ties all other structures together and is signed by an authorized
+            entity whose identity is represented by a trust anchor installed at the OPTIGA™. See Portected Update in the
+            Solution Reference Manual
+
+        :param fragments
+            Data streams used for the actual data update. fragments should be a list, where all the fragments are
+            sorted in the order they should be transmitted to the chip. Each fragment should be bytes
+
+        :raises
+            - ValueError - when any of the parameters contain an invalid value
+            - TypeError - when any of the parameters are of the wrong type
+            - OSError - when an error is returned by the chip initialisation library
+        """
+        api = self.api
+
+        if not isinstance(manifest, bytes) and not isinstance(manifest, bytearray):
+            raise TypeError("manifest should be bytes type")
+
+        if not isinstance(fragments, list) and not isinstance(fragments, list):
+            raise TypeError("fragments should be bytes type")
+
+        for fragment in fragments:
+            if not isinstance(fragment, bytes) and not isinstance(fragment, bytearray):
+                raise TypeError("Each fragment should be bytes type")
+
+        _backend.protected_update(api, manifest, fragments)
+
     def __str__(self):
         header = "=============================================="
         top = "Guessed chip name: {0}\n".format(self.name)
@@ -298,6 +316,7 @@ _meta_tags = {
     'execute': 0xd3,
     'change': 0xd0,
     'read': 0xd1,
+    'meta_update': 0xd8,
     'metadata': 0x20,
     'lcso': 0xc0,
     'version': 0xc1,
@@ -447,7 +466,7 @@ _data_object_types_swaped = {y: x for x, y in _data_object_types.items()}
 _reset_types = {
     # Setting the LcsO of either a key or data object.
     'lcso_to_creation': 0x01,
-    'lcso_to_initialisation': 0x02,
+    'lcso_to_initialisation': 0x03,
     'lcso_to_operational': 0x07,
     'lcso_to_termination': 0x0f,
     # - Flushing of either a key or data object with zero and set the used length of data objects, if present, to 0x0000
@@ -489,16 +508,13 @@ def _parse_access_conditions(tag_size, meta_itr):
         if _id in _access_conditions_ids_swaped:
             # Conf, Int, auto and luc have as the last two bytes a reference to the oid used for the expression
             # it is just another OID from the system
-            if _id == _access_conditions_ids['conf'] \
-                    or _id == _access_conditions_ids['int'] \
-                    or _id == _access_conditions_ids['auto'] \
-                    or _id == _access_conditions_ids['luc']:
+            if _id in (_access_conditions_ids['conf'], _access_conditions_ids['int'],
+                       _access_conditions_ids['auto'], _access_conditions_ids['luc']):
                 access_conditions.append(_access_conditions_ids_swaped[_id])
                 access_conditions.append(hex(next(meta_itr)))
                 access_conditions.append(hex(next(meta_itr)))
                 i += 2
-            elif _id == _access_conditions_ids['sec_sta_a'] \
-                    or _id == _access_conditions_ids['sec_sta_g']:
+            elif _id in (_access_conditions_ids['sec_sta_a'], _access_conditions_ids['sec_sta_g']):
                 access_conditions.append(_access_conditions_ids_swaped[_id])
                 access_conditions.append(hex(next(meta_itr)))
                 i += 1
@@ -515,6 +531,8 @@ def _parse_access_conditions(tag_size, meta_itr):
     return access_conditions
 
 
+# pylint: disable=unused-argument
+# we keep it for consistent API
 def _parse_lifecycle_state(tag_size, meta_itr):
     lcso = next(meta_itr)
     if lcso not in lifecycle_states:
@@ -524,6 +542,8 @@ def _parse_lifecycle_state(tag_size, meta_itr):
     return lifecycle_states[lcso]
 
 
+# pylint: disable=unused-argument
+# we keep it for consistent API
 def _parse_key_usage(tag_size: int, meta_itr) -> list:
     key_usage_bytes = next(meta_itr)
     tag_data = list()
@@ -539,6 +559,8 @@ def _parse_key_usage(tag_size: int, meta_itr) -> list:
     return tag_data
 
 
+# pylint: disable=unused-argument
+# we keep it for consistent API
 def _parse_algorithm(tag_size, meta_itr):
     algorithm = next(meta_itr)
     if algorithm not in _algorithms_swaped:
@@ -548,15 +570,31 @@ def _parse_algorithm(tag_size, meta_itr):
     return _algorithms_swaped[algorithm]
 
 
+# pylint: disable=unused-argument
+# we keep it for consistent API
 def _parse_reset_type(tag_size, meta_itr):
-    reset_type = next(meta_itr)
-    if reset_type not in _reset_types_swaped:
-        raise ValueError(
-            'Reset Type tag value {0} not found in supported {1}'.format(reset_type, _reset_types_swaped)
-        )
-    return _reset_types_swaped[reset_type]
+    reset_type_bytes = next(meta_itr)
+    tag_data = list()
+    reset_type_bytes_lower_nibble = reset_type_bytes & 0x0f
+    if reset_type_bytes_lower_nibble == _reset_types['lcso_to_initialisation']:
+        tag_data.append('lcso_to_initialisation')
+    if reset_type_bytes_lower_nibble == _reset_types['lcso_to_creation']:
+        tag_data.append('lcso_to_creation')
+    if reset_type_bytes_lower_nibble == _reset_types['lcso_to_operational']:
+        tag_data.append('lcso_to_operational')
+    if reset_type_bytes_lower_nibble == _reset_types['lcso_to_termination']:
+        tag_data.append('lcso_to_termination')
+
+    if reset_type_bytes & _reset_types['flushing']:
+        tag_data.append('flushing')
+    if reset_type_bytes & _reset_types['random_data']:
+        tag_data.append('random_data')
+
+    return tag_data
 
 
+# pylint: disable=unused-argument
+# we keep it for consistent API
 def _parse_type(tag_size, meta_itr):
     object_type = next(meta_itr)
     if object_type not in _data_object_types_swaped:
@@ -589,6 +627,7 @@ _parser_map = {
     'change': _parse_access_conditions,
     'execute': _parse_access_conditions,
     'read': _parse_access_conditions,
+    'meta_update': _parse_access_conditions,
     'version': _parse_version
 }
 
@@ -671,13 +710,13 @@ def _prepare_access_conditions(key, value: list) -> list:
     # we would like to skip some of values
     value_iter = iter(value)
     for element in value_iter:
-        if element == 'int' or element == 'conf' or element == 'auto' or element == 'luc':
+        if element in ('int', 'conf', 'auto', 'luc'):
             _meta = [
                 _access_conditions_ids[element],
                 int(next(value_iter), 16),
                 int(next(value_iter), 16),
             ]
-        elif element == 'sec_sta_g' or element == 'sec_sta_a':
+        elif element in ('sec_sta_g', 'sec_sta_a'):
             _meta = [
                 _access_conditions_ids[element],
                 int(next(value_iter), 16)
@@ -698,6 +737,7 @@ def _prepare_access_conditions(key, value: list) -> list:
     return meta
 
 
+# ToDo: Add a test to test a key usage assignment
 def _prepare_key_usage(key, value) -> int and list:
     key_usage = 0
     # the value should be of type list()
@@ -705,6 +745,7 @@ def _prepare_key_usage(key, value) -> int and list:
         raise TypeError(
             'key usage tag should be provided in the form of a list for instance [\'x\', \'y\', \'z\']'
         )
+
     for i in value:
         if i not in _key_usages:
             raise ValueError(
@@ -721,6 +762,30 @@ def _prepare_key_usage(key, value) -> int and list:
     return meta
 
 
+# ToDo: Add a test to test a reset type assignment
+def _prepare_reset_type(key, value) -> int and list:
+    reset_type = 0
+    # the value should be of type list()
+    if not isinstance(value, list):
+        raise TypeError(
+            'reset type tag should be provided in the form of a list for instance [\'x\', \'y\', \'z\']'
+        )
+    for i in value:
+        if i not in _reset_types:
+            raise ValueError(
+                'reset type isn\'t supported. Supported values {0}, you provided {1}'.format(_reset_types, i)
+            )
+        reset_type |= _reset_types[i]
+
+    meta = [
+        _meta_tags[key],  # key
+        1,  # size
+        reset_type  # value
+    ]
+
+    return meta
+
+
 def _prepare_lcso(key, value) -> list:
     if value not in _lifecycle_states_swaped:
         raise ValueError(
@@ -755,16 +820,18 @@ def _prepare_meta_and_size(key, value) -> list:
     # This is how the result should look like
     # key  size  value
     # Used size and max size tags can't be send to the chip, so ignore them with a warning
-    if key == 'used_size' or key == 'max_size' or key == 'algorithm':
+    if key in ('used_size', 'max_size', 'algorithm'):
         print('Tag \'{0}\' cannot be defined by a user. Skip.'.format(key))
         return list()
     # Parse each key, and construct a
-    elif key == 'type':
+    if key == 'type':
         meta = _prepare_type(key, value)
     elif key == 'lcso':
         meta = _prepare_lcso(key, value)
     elif key == 'key_usage':
         meta = _prepare_key_usage(key, value)
+    elif key == 'reset_type':
+        meta = _prepare_reset_type(key, value)
     # otherwise the value is most likely an access condition expression
     elif isinstance(value, list):
         meta = _prepare_access_conditions(key, value)
@@ -846,6 +913,8 @@ class Object:
     :type updated: bool
     """
 
+    # pylint: disable=invalid-name
+    # id is a valid name here
     def __init__(self, object_id):
         """
         This class
@@ -890,9 +959,9 @@ class Object:
         """
         if 'used_size' in self.meta:
             return self.meta['used_size']
-        else:
-            warnings.warn("Object doesn't have used_size property.")
-            return None
+
+        warnings.warn("Object doesn't have used_size property.")
+        return None
 
     @property
     def max_size(self):
@@ -900,9 +969,9 @@ class Object:
         """
         if 'max_size' in self.meta:
             return self.meta['max_size']
-        else:
-            warnings.warn("Object doesn't have max_size property.")
-            return None
+
+        warnings.warn("Object doesn't have max_size property.")
+        return None
 
     def read(self, offset=0, force=False) -> bytearray:
         """
